// Hartley.CPP
// Author  : Mike Nice 73565,565

#include <stdlib.h>
#include <math.h>
#include "hartley.hpp"

const double PI = 3.1415926535;




void FHT (FHTFunction function,  unsigned int numberOfDataPoints,
          const double inputTable[], double resultTable[])
{

  unsigned int testPower,
    i, j, k, l,
    ip, ip1, ip2,
    iMinus1, jMinus1,
    nMinus1, nOver2,
    le, le1, le2,
    m;  // 2 ^ m ==> number of data points

  double cos0, sin0, cos1, sin1;

  double temp, tempU;


  nMinus1 = numberOfDataPoints - 1;
  nOver2 = numberOfDataPoints / 2;

  // Determine the power of 2
  m = 1;
  testPower = 2;
  while (testPower < numberOfDataPoints) {
    testPower = testPower * 2;
    m++;
  } 

  // Perform array bit reversal while copying to resultTable array
  j = 1;
  for (i = 1; i<numberOfDataPoints; i++) {
    if (i <= j) {
      iMinus1 = i-1;
      jMinus1 = j-1;
      resultTable[iMinus1] = inputTable[jMinus1];
      resultTable[jMinus1] = inputTable[iMinus1];
    } 

    k = nOver2;
    while (k < j) {
      j -= k;
      k /= 2 ;
    } 
    j += k;

  } 

  // Copy last point manually
  resultTable[nMinus1] = inputTable[nMinus1];


  for (i = 0; i<numberOfDataPoints; i+=2) {
    temp = resultTable[i+1];
    resultTable[i+1] = resultTable[i] - temp;
    resultTable[i] += temp;
  } 

  le = 2;  // 2 ^ lth power
  for (l = 2; l<=m; l++) {
    le1 = le;
    le *= 2;  // Compute next power of 2
    le2 = le / 4;

    cos0 = cos(PI / (double)le1);
    sin0 = sin(PI / (double)le1);

    cos1 = cos0;
    sin1 = sin0;

    for (j=1; j<= numberOfDataPoints; j+= le) {
      for (i=j; i <= (j+le2); i+=le2) {
        ip = i + le1 - 1;
        iMinus1 = i-1;
        temp = resultTable[ip];
        resultTable[ip] = resultTable[iMinus1] - temp;
        resultTable[iMinus1] = resultTable[iMinus1] + temp;
      } 
    } 

    for (j = 2; j<=le2; j++) {
      ip1 = le + 1 - j;
      ip2 = ip1 - le1;

      for (i=j; i <= numberOfDataPoints; i+=le) {
        ip = i + le1 - 1;
        iMinus1 = i-1;
        temp = cos0 * resultTable[ip] +  sin0 * resultTable[ip1];
        tempU = sin0 * resultTable[ip] -  cos0 * resultTable[ip1];
        resultTable[ip] = resultTable[iMinus1] - temp;
        resultTable[iMinus1] = resultTable[iMinus1] + temp;
        resultTable[ip1] = resultTable[ip2] - tempU;
        resultTable[ip2] = resultTable[ip2] + tempU;
        ip1 = ip1 + le;
        ip2 = ip2 + le;

      } 

      temp = cos0*cos1 - sin0*sin1;
      sin0 = cos0*sin1 + cos1*sin0;
      cos0 = temp;

    } 

  } 


  if (function == calcFHT) {
    for (i = 0; i<numberOfDataPoints; i++) {
      resultTable[i] /= (double)numberOfDataPoints;
    } 
  } 


} // FHT



void SelectHarmonics(unsigned int firstHarmonic, unsigned int lastHarmonic,
                     unsigned int numberOfDataPoints,
                     double coeff[])
{

 unsigned int upperLimit, i;


  upperLimit = (numberOfDataPoints / 2) -1;

  if (firstHarmonic > 0) {
    coeff[0] = 0.0;
  } 

  if (firstHarmonic > 1) {
    for (i = 1; i<firstHarmonic; i++) {
      coeff[i] = 0.0;
      coeff[numberOfDataPoints - i] = 0.0;
    } 
  } 

  if (lastHarmonic < upperLimit) {
    for (i = ( lastHarmonic+1 ); i<=upperLimit; i++) {
      coeff[i] = 0.0;
      coeff[numberOfDataPoints - i] = 0.0;
    } 
  } 

  if (lastHarmonic < (numberOfDataPoints / 2))
    coeff[numberOfDataPoints /2] = 0.0;

} // SelectHarmonics



/**********************************************************************/
/*                 PROCEDURE  CalculateHarmonic                       */
/**********************************************************************/
void CalculateHarmonic(const double coeffArray[],
                       unsigned int numberOfDataPoints, 
                       unsigned int harmonic,
                       double & peakToPeak, double & angle)
{
  const double SquareRoot2Times2 = 2.828427125;

  double 
    realPart, imag,
    x, y,
    compensation;
  unsigned int modulus;

  realPart = coeffArray[harmonic];
  imag = coeffArray[numberOfDataPoints - harmonic];

  peakToPeak = sqrt((realPart * realPart) + (imag * imag)) * SquareRoot2Times2;

  y = realPart - imag;
  x = realPart + imag;

  if (x < 0.0)   // 2nd or 3rd quadrant
    compensation = 180.0;
  else
    compensation = 360.0;

  if (fabs(x) < 1.0E-32) {
    x = 1.0E-32;   // Small # to prevent divide by 0 overflow
  } 

  angle = (atan(y/x) * 180.0 / PI) + compensation;
  if (angle >= 360.0) angle -= 360.0;
  angle /= (double) harmonic;

} // CalculateHarmonic



