/*
Description : Test of Hartley transform module

  Hartley transforms allow faster calculations than a full
  complex FFT when working with only real data.  See
  Hartley.hpp for more details.

Author  : Mike Nice 73565,565
System  : IBM CSet++

*/


#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <math.h>
#include <sys\stat.h>
#include "hartley.hpp"

const double PI=3.1415926535897;

#define STDOUT 1
#define DataSize 1024

/* All except DF */
double dataTable[DataSize];
double coefTable [DataSize];
double resTable [DataSize];





void InitDataTable()
{

  const unsigned int NHarmonics=8;

  // Create waveform consisting of some sine components

  for (unsigned int i=0; i<DataSize; i++) {
    dataTable[i] = 0.0;
    for (unsigned int j=1; j<=NHarmonics; j++) {
      dataTable[i] += sin( (double)(i*j) * 2 * PI /(double)DataSize)
		     * (double) (DataSize / j);
    }
  }
} // InitDataTable



void WriteDataTable(unsigned int tableSize, const double table[],
		    char *fileName)
{
  FILE * file = fopen(fileName,"w");
  if (file == NULL) {
    perror(fileName);
  } else {
    for (unsigned int i=0; i<tableSize; i++)
      fprintf(file,"%1.3lf\n", table[i]);
    fclose(file);
  }
} // WriteDataTable


void WriteHarm(unsigned int number) {

  // Calculate requested harmonics
  printf("Harm  Peak-to-Peak  angle of peak\n");
  printf("----  ------------  -------------\n");
  for (unsigned int i=1; i<=number; i++) {

    double ptp, angle;
    CalculateHarmonic(coefTable, DataSize, i, ptp, angle);

    printf("  %2d      %8.3lf     %5.1lf\n", i, ptp, angle);
  }
} // WriteHarm



void TestCalcs()
{

  printf("Saving input data to INPUT.DAT...");
  WriteDataTable(DataSize, dataTable,"Input.DAT");

  printf("done.\n\n Calculating FHT...");

  FHT(calcFHT, DataSize, dataTable, coefTable);

  printf("complete.\n\n");

  WriteHarm(10);

  SelectHarmonics(1, 4, DataSize, coefTable);

  printf("\nCalculating Inverse FHT...");
  FHT(inverseFHT, DataSize, coefTable, resTable);

  printf("done.\nSaving filtered data to result.DAT...");
  WriteDataTable(DataSize, resTable,"Result.DAT");
  printf("done.\n\n");

  printf("Press a key to see filtered data..");
  getch();

  printf("\n\n");
  FHT(calcFHT, DataSize, resTable, coefTable);
  WriteHarm(5);


} // TestCalcs







main()
{
  struct stat statbuf;

  // IF output isn't redirected, make sure it is unbuffered
  if (fstat(STDOUT, &statbuf) == 0)
    if (statbuf.st_mode & S_IFCHR){
      setbuf(stdout, NULL);
      setbuf(stderr, NULL);
  }

  InitDataTable();

  TestCalcs();

  return (0);
} /* main */


